#!/usr/bin/env python3
import math, sys, os, random

random.seed('nac20')

if os.listdir():
    print('Error: directory not empty')
    sys.exit(1)

MAX_N = 200000
LOGN = 20

class TestCase(object):
    def __init__(self, edges, queries, expected=None):
        self.edges = edges
        self.queries = queries

        self.ans = self.solve()
        if expected is not None:
            assert self.ans == expected

    def solve(self):
        n = len(self.edges) + 1
        tree = [[] for _ in range(n)]
        for u, v in self.edges:
            tree[u].append(v)
            tree[v].append(u)

        st = [[-1 for _ in range(n)] for _ in range(LOGN)]
        depth = [-1 for _ in range(n)]
        stk = [0]
        depth[0] = 0
        while stk:
            u = stk.pop()
            for v in tree[u]:
                if depth[v] == -1:
                    depth[v] = depth[u] + 1
                    st[0][v] = u
                    stk.append(v)

        for j in range(LOGN - 1):
            for i in range(n):
                if st[j][i] != -1:
                    st[j + 1][i] = st[j][st[j][i]]

        def dist(u, v):
            if depth[u] > depth[v]:
                u, v = v, u

            orig_u, orig_v = u, v
            for j in range(LOGN - 1, -1, -1):
                if depth[v] - depth[u] >= (1 << j):
                    v = st[j][v]
            if u == v:
                return depth[orig_v] - depth[u]

            for j in range(LOGN - 1, -1, -1):
                if st[j][u] != st[j][v]:
                    u, v = st[j][u], st[j][v]

            return depth[orig_u] + depth[orig_v] - 2 * depth[st[0][u]]

        ans = []
        for u, v in self.queries:
            d = dist(u, v)
            ans.append(n + d * (d + 1) // 2)

        return ans

    def in_lines(self):
        return ['{} {}'.format(len(self.edges) + 1, len(self.queries))] + ['{} {}'.format(u + 1, v + 1) for u, v in self.edges] + ['{} {}'.format(u + 1, v + 1) for u, v in self.queries]

    def out_lines(self):
        return [str(x) for x in self.ans]

test_num = 0
def write_test(base_path, tc, name='', force_name=''):
    if not os.path.exists(base_path):
        os.makedirs(base_path)
    global test_num

    name = ('{:02d}'.format(test_num)) + ('_' + name if name else '')
    if force_name:
        name = force_name

    fn_in = os.path.join(base_path, name + '.in')
    with open(fn_in, 'w') as f_in:
        f_in.write(''.join(map(lambda x: x + '\n', tc.in_lines())))
    fn_ans = os.path.join(base_path, name + '.ans')
    with open(fn_ans, 'w') as f_ans:
        f_ans.write(''.join(map(lambda x: x + '\n', tc.out_lines())))
    test_num += 1

def sample_1():
    edges = [(0, 1), (1, 2), (1, 3), (3, 4)]
    queries = [(0, 2), (3, 4)]
    return TestCase(edges, queries, [8, 6])

def single_node():
    edges = []
    queries = [(0, 0)]
    return TestCase(edges, queries, [1])

def single_edge():
    edges = [(0, 1)]
    queries = [(0, 0), (1, 1), (0, 1)]
    return TestCase(edges, queries, [2, 2, 3])


def hand_1():
    edges = [(0, 1), (0, 2), (1, 3), (1, 4), (2, 5)]

    return TestCase(taxes, edges)

def star(n):
    labels = list(range(n))
    random.shuffle(labels)
    edges = [(labels[0], labels[i]) for i in range(1, n)]
    random.shuffle(edges)

    queries = []
    for i in range(2, n, 2):
        queries.append((labels[0], labels[i]))
    for i in range(3, n, 2):
        queries.append((labels[i - 2], labels[i]))

    return TestCase(edges, queries)

def linked_list(n):
    labels = list(range(n))
    random.shuffle(labels)
    edges = [(labels[i - 1], labels[i]) for i in range(1, n)]
    random.shuffle(edges)

    cap = int(math.sqrt(n) / 2.0)
    queries = []
    for i in range(cap):
        for j in range(cap):
            queries.append((labels[i], labels[-j - 1]))

    for _ in range(n // 2):
        queries.append((random.choice(labels), random.choice(labels)))

    return TestCase(edges, queries)

def width_tree(n, width):
    labels = list(range(n))
    random.shuffle(labels)
    edges = []
    for i in range(1, n):
        parent = (i - 1) // width
        edges.append((labels[parent], labels[i]))
    random.shuffle(edges)

    queries = [(random.choice(labels), random.choice(labels)) for _ in range(n)]

    return TestCase(edges, queries)

write_test('sample', sample_1(), 'sample_1', '01')

# tiny
write_test('secret', single_node(), 'single_node')
write_test('secret', single_edge(), 'single_edge')

# small
write_test('secret', star(20), 'star_' + str(20))
write_test('secret', linked_list(20), 'linked_list_' + str(20))

for w in [2, 250]:
    write_test('secret', width_tree(250, w), 'tree_width{}_{}'.format(w, 250))

# medium
write_test('secret', star(2000), 'star_' + str(2000))
write_test('secret', linked_list(2000), 'linked_list_' + str(2000))

# venti!
write_test('secret', star(MAX_N), 'star_' + str(MAX_N))
write_test('secret', linked_list(MAX_N), 'linked_list_' + str(MAX_N))

for w in [2, 100]:
    write_test('secret', width_tree(MAX_N, w), 'tree_width{}_{}'.format(w, MAX_N))
